/*
	$Id: Body.java 6881 2015-04-21 07:59:42Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.extensions.DynamicActionAttributeSupport;
import net.sf.basedb.clients.web.extensions.skin.SkinAction;
import net.sf.basedb.clients.web.util.HTML;
import net.sf.basedb.core.Application;
import net.sf.basedb.core.SessionControl;
import net.sf.basedb.util.Values;

import java.util.Date;
import java.util.List;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.BodyContent;

/**
	This tag defines the body section of a page. It must be located inside
	a {@link Page &lt;base:page&gt;} ag.

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:body
      id=...
      clazz=...
      subclass=...
      style=...
   &gt;
</pre>

	<table border=1 cellspacing=0 cellpadding=2>
	<caption>Tag attributes</caption>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>no</td>
		<td>The ID of the body tag. The value of this attribute
		goes directly into the standard <code>id</code> attribute.
		This means that if you need a reference to the button object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		note = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>clazz</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>class</code> attribute. This allows you to apply different styles
			to different pages. The default class depends on the page type: iframe, 
			popup, default.
		</td>
	</tr>
	<tr>
		<td>subclass</td>
		<td>-</td>
		<td>no</td>
		<td>
			Additional classes to put in the HTML class attribute. Doesn't
			replace the main class attribute.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tables.
		</td>
	</tr>
	</table>
	
	@author Nicklas
	@version 2.0
*/
public class Body
	extends StylableTag
{
	
	/**
		
	*/
	private static final long serialVersionUID = 6814973550614834311L;

	public Body()
	{
		// The default class is different depending on the page type
		super(null);
	}
	
	/**
		The parent &lt;base:page&gt; tag.
	*/
	private Page page;

	/*
		Taglib initialization methods
	*/
	@Override
	public String getClazz() 
	{
		String pageClass = super.getClazz();
		if (pageClass == null && page != null)
		{
			pageClass = getDefaultClazz(page.getTypeCode());
		}
		return pageClass;
	}
	public String getDefaultClazz(int pageType)
	{
		String defaultClazz;
		switch (pageType)
		{
			case Page.PAGE_TYPE_IFRAME:
				defaultClazz = "iframe";
				break;
			case Page.PAGE_TYPE_POPUP:
				defaultClazz = "popup";
				break;
			default:
				defaultClazz = "default";
		}
		return defaultClazz;
	}
	
	@Override
	public int doStartTag()
		throws JspException
	{
		if (!(getParent() instanceof Page)) throw new JspTagException("Tag <base:body> must be inside a <base:page> tag");
		page = (Page)getParent();
		SessionControl sc = page.getSessionControl();
		int pageType = page.getTypeCode();
	
		StringBuilder sb = new StringBuilder();
		
		if (pageType != Page.PAGE_TYPE_INCLUDE)
		{
			sb.append("<body");
			addIdAndStyles(sb);
			sb.append(" data-app-root=\"").append(page.getRoot()).append("\"");
			sb.append(" data-gui-scale=\"").append(Base.getScale(sc)).append("\"");
			String longTexts = Values.getString(sc != null ? sc.getUserClientSetting("text.long") : null, "display");
			sb.append(" data-text-overflow=\"").append(longTexts).append("\"");
		
			String message = pageContext.getRequest().getParameter("popmessage");
			if (sc != null)
			{
				sb.append(" data-session-id=\"").append(sc.getId()).append("\"");
				if (sc.getLoggedInUserId() != 0)
				{
					sb.append(" data-user-id=\"").append(sc.getLoggedInUserId()).append("\"");
				}
				if (sc.getActiveProjectId() != 0)
				{
					sb.append(" data-project-id=\"").append(sc.getActiveProjectId()).append("\"");
				}
				sb.append(" data-remember-dialog-positions=\"").append(Values.getString(sc.getUserClientSetting("dialogs.remember-positions"), "1")).append("\"");
				if (message == null)
				{
					message = (String)sc.getSessionSetting("alert-message");
					sc.setSessionSetting("alert-message", null);
				}
			}
			if (message != null)
			{
				sb.append(" data-alert-message=\"").append(HTML.encodeTags(message)).append("\"");
			}
			if (page.getMaxUrlLength() != null)
			{
				sb.append(" data-max-url-length=\"").append(page.getMaxUrlLength()).append("\"");
			}
			addDynamicAttributes(sb);
			sb.append(">\n");
			
			// Output <div> tag with skin extension dynamic attributes
			List<SkinAction> skinActions = page.getSkinActions();
			if (skinActions != null && skinActions.size() > 0)
			{
				sb.append("<div id=\"skin-data\" class=\"datacontainer\">\n");
				for (SkinAction skin : skinActions)
				{
					if (skin.getId() != null)
					{
						sb.append("\t<div id=\"").append(skin.getId()).append("\"");
						sb.append(DynamicActionAttributeSupport.getAttributesString(skin));
						sb.append("></div>\n");
					}
				}
				sb.append("</div>");
			}
		}
		
		
		if (pageType == Page.PAGE_TYPE_DEFAULT)
		{
			// Include menu on "default" pages
			String menuName = page.getMenu();
			if ("auto".equals(menuName)) menuName = sc != null && sc.isLoggedIn() ? "standard" : "login";
			String menuHtml = null;
			if (sc != null && pageContext.getRequest().getParameter("refresh_menu") == null)
			{
				// The menu cache can be used, but check expiration date
				Date expirationDate = (Date)sc.getSessionSetting("menu."+menuName+".expire");
				if (expirationDate != null && expirationDate.after(new Date()))
				{
					menuHtml = (String)sc.getSessionSetting("menu."+menuName+".html");
				}
			}
			if (menuHtml == null)
			{
				try
				{
					/*
					We don't want the include() method to flush the pageContext.getOut() stream
					*/
					BodyContent menuBody = pageContext.pushBody();
					pageContext.include("/include/menu.jsp?name="+menuName);
					pageContext.popBody();
					menuHtml = menuBody.getString();
					if (sc != null) 
					{
						sc.setSessionSetting("menu."+menuName+".html", menuHtml);
						sc.setSessionSetting("menu."+menuName+".expire", 
							new Date(System.currentTimeMillis()+60000L*Application.permissionTimeout())); 
					}
				}
				catch (Exception ex)
				{
					throw new JspTagException(ex);
				}
			}
			if (menuHtml != null) sb.append(menuHtml);
			try
			{
				BodyContent menuBody = pageContext.pushBody();
				pageContext.include("/include/menu.jsp?name=recentItems");
				pageContext.include("/include/menu.jsp?name=projects");
				pageContext.popBody();
				sb.append(menuBody.getString());
			}
			catch (Exception ex)
			{
				throw new JspTagException(ex);				
			}
		}
		
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex);
		}
		return EVAL_BODY_INCLUDE;
	}

	@Override
	public int doEndTag() 
		throws JspException
	{
		if (page.getTypeCode() != Page.PAGE_TYPE_INCLUDE)
		{
			StringBuilder sb = new StringBuilder();
			sb.append("</body>\n");
			try
			{
				pageContext.getOut().print(sb.toString());
			}
			catch (Exception ex)
			{
				throw new JspTagException(ex.getMessage());
			}
		}
		return EVAL_PAGE;
	}
}



