/**
	$Id $

	Copyright (C) 2014 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.List;

import net.sf.basedb.core.authentication.AuthenticatedUser;
import net.sf.basedb.core.authentication.LoginRequest;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.core.query.QueryParameter;
import net.sf.basedb.util.extensions.ClientContext;

/**
	Gives authentication implementations limited access to the database.

	@author Nicklas
	@since 3.3
	@base.modified $Date $
*/
public class AuthenticationContext
	extends ClientContext
{

	private final org.hibernate.Session session;
	private final LoginRequest loginRequest;
	
	AuthenticationContext(SessionControl sc, org.hibernate.Session session, LoginRequest loginRequest)
	{
		super(sc, null, loginRequest);
		this.session = session;
		this.loginRequest = loginRequest;
	}
	
	/**
		Get the login request information.
	*/
	public LoginRequest getLoginRequest()
	{
		return loginRequest;
	}
	
	/**
		Load a user item from the BASE database given a login name.
		@param login The login for the user
		@return A UserData object or null if there is no user
	*/
	public UserData getUserByLogin(String login)
	{
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, "GET_USER_FOR_LOGIN");
		/*
			SELECT usr
			FROM UserData usr
			WHERE usr.login = :login
		*/
		query.setString("login", login);
		return HibernateUtil.loadData(UserData.class, query);
	}
	
	/**
		Load a user item from the BASE database given an external id.
		@param id The external id for the user
		@return A UserData object or null if there is no user
	*/
	public UserData getUserByExternalId(String id)
	{
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, "GET_USER_FOR_EXTERNAL_ID");
		/*
			SELECT usr 
			FROM UserData usr
			WHERE usr.externalId = :externalId
		*/
		query.setParameter("externalId", id);
		return HibernateUtil.loadData(UserData.class, query);
	}
	
	/**
		Find users in the database using a "free" restriction. The restriction
		may contain named parameters but their values must be supplied
		as {@link QueryParameter} objects. 
		
		IMPORTANT! Do not insert user-data into the restriction string since
		it may open up for SQL injection attacks.
		
		@param restriction A HQL restriction put in the WHERE clause
		@param parameters Optional query parameters
		@return A list of users (may be empty)
	*/
	public List<UserData> findUsers(String restriction, QueryParameter... parameters)
	{
		String sql = "SELECT usr FROM UserData usr";
		if (restriction != null)
		{
			sql += " WHERE " + restriction;
		}
		org.hibernate.Query query = HibernateUtil.createQuery(session, sql);
		if (parameters != null)
		{
			for (QueryParameter qp : parameters)
			{
				query.setParameter(qp.getName(), qp.getValue(), qp.getType().getTypeWrapper().getHibernateType());
			}
		}
		return HibernateUtil.loadList(UserData.class, query, getSessionControl());
	}
	
	/**
		Load a user item from the BASE database given an internal id.
		@param id The internal id for the user
		@return A UserData object or null if there is no user
	*/
	public UserData getUserById(int id)
	{
		return HibernateUtil.loadData(session, UserData.class, id);
	}
	
	/**
		Check that the user exists and verify the password using
		internal authentication.
	*/
	public AuthenticatedUser verifyUserInternal(LoginRequest loginRequest)
	{
		return getSessionControl().verifyUserInternal(session, loginRequest);
	}

	
}
