/*
 $Id: Protocol.java 7308 2017-03-13 08:15:46Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Date;
import java.util.Set;

import net.sf.basedb.core.data.ProtocolData;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
 	This class is used to represent individual protocol items
	and information about them.

	@author enell
	@version 2.0
*/
public class Protocol
	extends AnnotatedItem
	implements FileAttachable, Registered, Subtypable
{
	/**
		The type of item represented by this class.
		@see Item#PROTOCOL
		@see #getType()
	*/
	public static final Item TYPE = Item.PROTOCOL;

	/**
		The ID for the sampling protocol type, for example a protocol
		used when creating a {@link Sample} from a {@link
		BioSource}.
	 	@since 3.0
	*/
	public static final String SAMPLING = "net.sf.basedb.core.ProtocolType.SAMPLING";
	
	/**
		The ID for the extraction protocol type, for example a protocol
		used when creating an {@link Extract} from a {@link
		Sample}.
	 	@since 3.0
	*/
	public static final String EXTRACTION = "net.sf.basedb.core.ProtocolType.EXTRACTION";
	
	/**
		The ID for the labeling protocol type, for example a protocol
		used when creating a labeled extract from an {@link Extract}.
	 	@since 3.0
	*/
	public static final String LABELING = "net.sf.basedb.core.ProtocolType.LABELING";

	/**
		The ID for the library preparation protocol type, for example a protocol
		used when creating a library extract from an {@link Extract}.
	 	@since 3.0
	*/
	public static final String LIBRARY_PREPARATION = "net.sf.basedb.core.ProtocolType.LIBRARY_PREPARATION";

	/**
		The ID for the pooling protocol type, for example a protocol
		used when creating a biomaterial item ({@link Sample} or
		{@link Extract}) from a set of other biomaterial items.
	 	@since 3.0
	*/
	public static final String POOLING = "net.sf.basedb.core.ProtocolType.POOLING";
	
	/**
		The ID for the hybridization protocol type, for example a protocol
		used when creating a {@link PhysicalBioAssay} from a set of
		(labeled) {@link Extract}:s.
	 	@since 3.0
	*/
	public static final String HYBRIDIZATION = "net.sf.basedb.core.ProtocolType.HYBRIDIZATION";

	/**
		The ID for the cluster generation (in a flow cell) protocol type, for example 
		a protocol used when creating a {@link PhysicalBioAssay}.
	 	@since 3.0
	*/
	public static final String CLUSTER_GENERATION = "net.sf.basedb.core.ProtocolType.CLUSTER_GENERATION";

	/**
		The ID for the sequencing protocol type, for example a protocol
		used when creating a {@link DerivedBioAssay} from a {@link PhysicalBioAssay}
	 	@since 3.0
	*/
	public static final String SEQUENCING = "net.sf.basedb.core.ProtocolType.SEQUENCING";

	/**
		The ID for the alignment protocol type, for example a protocol
		used when creating a {@link DerivedBioAssay} representing data that has been 
		aligned against a reference genome.
	 	@since 3.0
	*/
	public static final String ALIGNMENT = "net.sf.basedb.core.ProtocolType.ALIGNMENT";
	
	/**
		The ID for the printing protocol type, for example a protocol
		used when printing a batch of array slides using a print robot.
	 	@since 3.0
	*/
	public static final String PRINTING = "net.sf.basedb.core.ProtocolType.PRINTING";
	
	/**
		The ID for the scanning protocol type, for example a protocol
		used for scanning the microarray slide after an experiment.
	 	@since 3.0
	*/
	public static final String SCANNING = "net.sf.basedb.core.ProtocolType.SCANNING";
	
	/**
		The ID for the feature extraction protocol type, for example a
		protocol used for analysing one or more images to create a raw
		data file.
	 	@since 3.0
	*/
	public static final String FEATURE_EXTRACTION = "net.sf.basedb.core.ProtocolType.FEATURE_EXTRACTION";
	
	/**
		The maximum length of the external id that can be stored in the database.
		@see #setExternalId(String)
		@since 2.15
	*/
	public static final int MAX_EXTERNAL_ID_LENGTH = ProtocolData.MAX_EXTERNAL_ID_LENGTH;

	/**
		Get the system id for the protocol subtype that should be used on an item
		that doesn't have a subtype. This includes {@link Subtypable} items with a
		null {@link ItemSubtype} and all other non-subtypable items which can be
		linked to a protocol.
		@param itemType The item type
		@return The system id of the default protocol or null
		@since 3.0
	*/
	public static String getDefaultSystemId(Item itemType)
	{
		String defaultId = null;
		if (itemType == Item.SAMPLE)
		{
			defaultId = SAMPLING;
		}
		else if (itemType == Item.EXTRACT)
		{
			defaultId = EXTRACTION;
		}
		else if (itemType == Item.ARRAYBATCH)
		{
			defaultId = PRINTING;
		}
		else if (itemType == Item.RAWBIOASSAY)
		{
			defaultId = FEATURE_EXTRACTION;
		}
		return defaultId;
	}
	
	/**
		Create a new <code>Protocol</code> item.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access
		@return The new <code>Protocol</code> item
		@throws BaseException If there is an error
	*/
	public static Protocol getNew(DbControl dc)
		throws  BaseException
	{
		Protocol p = dc.newItem(Protocol.class);
		p.setName("New protocol");
		p.getData().setEntryDate(new Date());
		return p; 
	}
	
	/**
		Get a <code>Protocol</code> item when you know the ID.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>Protocol</code> item
		@throws ItemNotFoundException This exception is thrown if an
			item with the specified ID is not found
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have {@link Permission#READ READ}
			permission to the item
		@throws BaseException This exception is thrown if there is another
			error
	*/
	public static Protocol getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		Protocol pr = dc.loadItem(Protocol.class, id);
		if (pr == null) throw new ItemNotFoundException("Protocol[ID="+id+"]");
		return pr;
	}
	
	/**
		Get a query configured to retrieve protocols.
		
		@return An {@link ItemQuery} object
	*/
	public static ItemQuery<Protocol> getQuery()
	{
		return new ItemQuery<Protocol>(Protocol.class);
	}
	
	/**
		Creates a new protocol item.
		
		@param data the data
	*/
	Protocol(ProtocolData data)
	{
		super(data);
	}
	
	@Override
	ProtocolData getData()
	{
		return (ProtocolData)getBasicData();
	}
	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the FileAttachable interface
		-------------------------------------------
	*/
	@Override
	public File getFile()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(File.class, getData().getFile());
	}

	@Override
	public void setFile(File file)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		if (file != null) file.checkPermission(Permission.USE);
		getData().setFile(file == null ? null : file.getData());
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	@Override
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	@Override
	public void setEntryDate(Date entryDate)
	{
		if (isInDatabase()) 
		{
			throw new PermissionDeniedException("Not allowed to change entryDate of " + this);
		}
		getData().setEntryDate(DateUtil.setDateOrToday(entryDate));
	}
	// -------------------------------------------
	/*
		From the Annotatable interface
		-------------------------------------------
	*/
	/**
		@return Always null
		@since 2.12
	*/
	@Override
	public Set<Annotatable> getAnnotatableParents()
	{
		return null;
	}
	// ---------------------------------------------
	/*
		From the Subtypable interface
		-----------------------------
	*/
	@Override
	@SubtypableRelatedItems({Item.FILE})
	public ItemSubtype getItemSubtype()
	{
		return getDbControl().getItem(ItemSubtype.class, getData().getItemSubtype());
	}
	@Override
	public void setItemSubtype(ItemSubtype subtype)
	{
		checkPermission(Permission.WRITE);
		if (subtype != null)
		{
			subtype.setOnItem(this);
		}
		else
		{
			getData().setItemSubtype(null);
		}
	}
	// -------------------------------------------
	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Check if this protocol is used by:
		<ul>
		<li>a {@link BioMaterialEvent}
		<li>a {@link PlateEvent}
		<li>an {@link ArrayBatch}
		<li>a {@link DerivedBioAssay}
		<li>a {@link RawBioAssay}
		</ul>
	*/
	@Override
	public boolean isUsed()
		throws BaseException
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		boolean used = false;
		
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOMATERIALEVENTS_FOR_PROTOCOL", "count(*)");
			/*
				SELECT {1}
				FROM BioMaterialEventData bme
				WHERE bme.protocol = :protocol
			*/
			query.setEntity("protocol", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_PLATEEVENTS_FOR_PROTOCOL", "count(*)");
			/*
				SELECT {1}
				FROM PlateEventData pe
				WHERE pe.protocol = :protocol
			*/
			query.setEntity("protocol", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_ARRAYBATCHES_FOR_PROTOCOL", "count(*)");
			/*
				SELECT {1}
				FROM ArrayBatchData ab
				WHERE ab.protocol = :protocol
			*/
			query.setEntity("protocol", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_DERIVEDBIOASSAY_FOR_PROTOCOL", "count(*)");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.protocol = :protocol
			*/
			query.setEntity("protocol", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		if (!used)
		{
			org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_RAWBIOASSAYS_FOR_PROTOCOL", "count(*)");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.protocol = :protocol
			*/
			query.setEntity("protocol", this.getData());
			used = HibernateUtil.loadData(Long.class, query) > 0;
		}
		return used || super.isUsed();
	}
	/**
		Get all:
		<ul>
		<li>a {@link BioMaterial}:s and {@link PhysicalBioAssay}:s using this protocol
		<li>a {@link Plate}:s using this protocol
		<li>an {@link ArrayBatch}:s using this protocol
		<li>a {@link DerivedBioAssay}:s using this protocol
		<li>a {@link RawBioAssay}:s using this protocol
		</ul>
		@since 2.2
	*/
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		
		// Biomaterial events - biomaterials
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
			"GET_BIOMATERIALEVENTS_FOR_PROTOCOL", "bme.bioMaterial");
			/*
				SELECT {1}
				FROM BioMaterialEventData bme
				WHERE bme.protocol = :protocol
			*/
		query.setEntity("protocol", this.getData());
		addUsingItems(using, query);

		// Biomaterial events - hybridizations
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_BIOMATERIALEVENTS_FOR_PROTOCOL", "bme.physicalBioAssay");
			/*
				SELECT {1}
				FROM BioMaterialEventData bme
				WHERE bme.protocol = :protocol
			*/
		query.setEntity("protocol", this.getData());
		addUsingItems(using, query);
		
		// Plate events
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_PLATEEVENTS_FOR_PROTOCOL", "pe.plate.id");
			/*
				SELECT {1}
				FROM PlateEventData pe
				WHERE pe.protocol = :protocol
			*/
		query.setEntity("protocol", this.getData());
		addUsingItems(using, Item.PLATE, query);
		
		// Array batches
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_ARRAYBATCHES_FOR_PROTOCOL", "ab.id");
			/*
				SELECT {1}
				FROM ArrayBatchData ab
				WHERE ab.protocol = :protocol
			*/
		query.setEntity("protocol", this.getData());
		addUsingItems(using, Item.ARRAYBATCH, query);
		
		// Scans
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_DERIVEDBIOASSAY_FOR_PROTOCOL", "dba.id");
			/*
				SELECT {1}
				FROM DerivedBioAssayData dba
				WHERE dba.protocol = :protocol
			*/
		query.setEntity("protocol", this.getData());
		addUsingItems(using, Item.DERIVEDBIOASSAY, query);
		
		// Raw bioassays
		query = HibernateUtil.getPredefinedQuery(session, 
			"GET_RAWBIOASSAYS_FOR_PROTOCOL", "rba.id");
			/*
				SELECT {1}
				FROM RawBioAssayData rba
				WHERE rba.protocol = :protocol
			*/
		query.setEntity("protocol", this.getData());
		addUsingItems(using, Item.RAWBIOASSAY, query);
		return using;
	}
	// -------------------------------------------

	/**
		Get the external id of the protocol. This value can be used to link
		with information in external databases. It is not used by the BASE core
		and it doesn't have to be unique.
		@return The external id for the protocol
		@since 2.15
	*/
	public String getExternalId()
	{
		return getData().getExternalId();
	}
	/**
		Set the external id of the protocol. The value
		may be null but must not be longer than the value specified by
		the {@link #MAX_EXTERNAL_ID_LENGTH} constant.
		@param externalId The new value for the external id
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@throws InvalidDataException If the string is too long
		@since 2.15
	*/
	public void setExternalId(String externalId)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExternalId(StringUtil.setNullableString(externalId, "externalId", MAX_EXTERNAL_ID_LENGTH));
	}

	
	/**
		Add an annotation type as a parameter to this protocol. 
		@param annotationType The annotation type to add to this protocol
		@throws PermissionDeniedException If the
			logged in user doesn't have {@link Permission#WRITE}
			permission for the protocol and {@link Permission#USE}
			for the annotation type
		@throws InvalidDataException If the annotationType is null, or the
			{@link AnnotationType#isProtocolParameter()} doesn't return true
		@since 2.2
	*/
	public void addParameter(AnnotationType annotationType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		annotationType.checkPermission(Permission.USE);
		if (!annotationType.isProtocolParameter())
		{
			throw new InvalidDataException("Annotation type '" + annotationType + "' is not a protocol parameter");
		}
		getData().getParameters().add(annotationType.getData());
	}
	
	/**
		Remove an annotation type from the parameters for this protocol.
		@param annotationType The annotation type to remove from this protocol
		@throws PermissionDeniedException If the
			logged in user doesn't have {@link Permission#WRITE}
			permission for the protocol
		@throws InvalidDataException If the annotationType is null
		@since 2.2
	*/
	public void removeParameter(AnnotationType annotationType)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		if (annotationType == null) throw new InvalidUseOfNullException("annotationType");
		getData().getParameters().remove(annotationType.getData());
	}
	
	/**
		Check if the annotation type is a parameter to this protocol.
		@param annotationType The annotation type to check, null always return false
		@return TRUE if the annotation type is a parameter, FALSE otherwise
		@since 2.2
		@see AnnotationType#isProtocolParameter(Protocol)
	*/
	public boolean isParameter(AnnotationType annotationType)
	{
		return annotationType == null ? 
			false : getData().getParameters().contains(annotationType.getData());
	}
	
	/**
		Check if the annotation is parameter to this protocol. This method
		is equivalent to {@link #isParameter(AnnotationType)} but is not
		affected by possible permission problems with the annotation type.
		@param annotation The annotation to check, null always return false
		@return TRUE if the annotation is a parameter, FALSE otherwise
		@since 2.10
	*/
	public boolean isParameter(Annotation annotation)
	{
		return annotation == null ? false : 
			getData().getParameters().contains(annotation.getData().getAnnotationType());
	}
	
	/**
	 	Get a query that returns all annotation types used as parameters 
	 	for this protocol.
		@return An <code>ItemQuery</code> object
		@since 2.2
	*/
	public ItemQuery<AnnotationType> getParameters()
	{
		ItemQuery<AnnotationType> query = AnnotationType.getQuery(null);
		query.joinPermanent(Hql.innerJoin("protocols", Item.PROTOCOL.getAlias()));
		query.restrictPermanent(
			Restrictions.eq(
				Hql.alias(Item.PROTOCOL.getAlias()), 
				Hql.entity(this)
			)
		);
		return query;
	}

}
