/**
	$Id: BaseFileParser.java 5590 2011-03-16 11:48:47Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.basefile;

import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.regex.Pattern;

import net.sf.basedb.core.AbsoluteProgressReporter;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.parser.FlatFileParser;

/**
	Parser for serial and matrix BASEfile:s. This class will setup and
	perform the line-by-line parsing, but it doesn't handle the data.
	Data is handled section-wise by registering {@link BaseFileSectionParser}:s
	with {@link #setSectionParser(String, BaseFileSectionParser)}.
	Sections that doens't have a registered parser are skipped.

	@author Nicklas
	@version 2.14
	@base.modified $Date: 2011-03-16 12:48:47 +0100 (on, 16 mar 2011) $
*/
public class BaseFileParser
{

	private final Map<String, BaseFileSectionParser> parsers;
	private AbsoluteProgressReporter progress;
	private Map<String, Integer> sectionCount;
	private Map<String, String> redefinedColumnNames;

	/**
		Creates a new parser object.
	*/
	public BaseFileParser()
	{
		this.parsers = new HashMap<String, BaseFileSectionParser>();
		this.sectionCount = new HashMap<String, Integer>();
		this.redefinedColumnNames = new HashMap<String, String>();
	}
	
	/**
		Adds a section parser. If a parser already exists for the
		given section it is replaced.
		
		@param section The name of the section the parser handles
		@param parser The parser, or null to remove a previously
			registered parser
	*/
	public void setSectionParser(String section, BaseFileSectionParser parser)
	{
		if (section == null) throw new NullPointerException("section");
		if (parser == null)
		{
			parsers.remove(section);
		}
		else
		{
			parsers.put(section, parser);
		}
	}
	
	/**
		Get the parser that is currently registered for a section.
		@param section The name of the section
		@return A section parser, or null if no parser is registered
	*/
	public BaseFileSectionParser getSectionParser(String section)
	{
		return parsers.get(section);
	}
	
	/**
		Redefined a column name for a specified section. This function is useful
		to configure parsers that normally look for, for example, the column
		'reporter' to instead look for 'Reporter ID'.
		@param section The section the redefined name is valid for
		@param defaultName The default name (eg. 'reporter')
		@param redefinedName The redefined name (ef. 'Reporter ID')
	*/
	public void setRedefinedColumnName(String section, String defaultName, String redefinedName)
	{
		redefinedColumnNames.put(section + ":" + defaultName, redefinedName);
	}
	
	/**
		Get the redefined column name for a section. If the name has not
		been redefined, the default name is returned.
		@param section The section the redefined name is valid for
		@param defaultName The default name
		@return The redefined name, or the default name if it has not
			been redefined
	*/
	public String getRedefinedColumnName(String section, String defaultName)
	{
		String redefinedName = redefinedColumnNames.get(section + ":" + defaultName);
		return redefinedName == null ? defaultName : redefinedName;
	}
	
	/**
		Copy redefined column names from the given parser into this
		parser. Column names that have already been redefined in this
		parser will be overwritten only if they have also been redefined in
		the other parser.
		@param parser The parser to copy redefined column names from
	*/
	public void copyRedefinedColumnNames(BaseFileParser parser)
	{
		redefinedColumnNames.putAll(parser.redefinedColumnNames);
	}
	
	/**
		Set a progress reporter that will be used to report 
		how the parsing is progressing.
		@param progress A progress reporter, or null to not report progress
	*/
	public void setProgressReporter(AbsoluteProgressReporter progress)
	{
		this.progress = progress;
	}
	
	/**
		Parse the given input stream.
		@param in The data stream to parse
		@param charset The character set used by the data, or null
			to use the configured default
		@return The FlatFileParser the was used to parse the BASEfile
		@throws IOException If there is any problem with the parsing
	*/
	public FlatFileParser parse(InputStream in, String charset)
		throws IOException
	{
		FlatFileParser ffp = getInitializedFlatFileParser(in, charset);
		while (ffp.hasMoreSections())
		{
			ThreadSignalHandler.checkInterrupted();
			FlatFileParser.Line section = ffp.nextSection();			
			increaseSectionCount(section.name(), 1);
			BaseFileSectionParser parser = getSectionParser(section.name());
			if (parser != null)
			{
				parser.parseSection(this, ffp);
			}
		}
		return ffp;
	}
	
	/**
		The number of times we have seen a certain section in the
		file.
		@param section The name of the section
	*/
	public int getSectionCount(String section)
	{
		Integer cnt = sectionCount.get(section);
		return cnt == null ? 0 : cnt;
	}
	
	/**
		Counts the number of times a section has been seen in the file.
		@param section The name of the section
		@param count The count to add to the current count
	*/
	protected void increaseSectionCount(String section, int count)
	{
		Integer cnt = sectionCount.get(section);
		if (cnt == null) cnt = 0;
		sectionCount.put(section, cnt + count);
	}
	
	/**
		Update the progress of the parsing.
		@see ProgressReporter
	*/
	public void setProgress(long completed, String message)
	{
		if (progress != null)
		{
			progress.displayAbsolute(completed, message);
		}
	}
	
	/**
		Creates a {@link FlatFileParser} for parsing a BASEfile.
		
		@param stream The stream that the parser should read from.
		@return The initialised parser
	*/
	protected FlatFileParser getInitializedFlatFileParser(InputStream stream, String charset)
	{
		FlatFileParser ffp = new FlatFileParser();
		ffp.setSectionRegexp(Pattern.compile("section[\\t ](.*)"));
		ffp.setHeaderRegexp(Pattern.compile("(.*?)\\t(.*)"));
		ffp.setDataHeaderRegexp(Pattern.compile("%"));
		ffp.setDataFooterRegexp(Pattern.compile("^$"));
		ffp.setDataSplitterRegexp(Pattern.compile("\\t"));
		ffp.setUseNullIfEmpty(false);
		ffp.setInputStream(stream, charset);
		return ffp;
	}
	
	/**
		Get the value of a required header. This method calls
		{@link FlatFileParser#getHeader(String)} to get the value of
		the header. If the header is missing or has an empty value
		an exception is thrown.
		
		@param ffp The flat file parser
		@param header The name of the header
		@param section The name of the section that is being parsed
		@param filename The name of the file that is being parsed (used to 
			create the error message in case the header is missing)
		@return The value of the header
	*/
	public String getRequiredHeader(FlatFileParser ffp, String header, 
		String section, String filename)
	{
		String value = Values.getStringOrNull(ffp.getHeader(header));
		if (value == null)
		{
			throw new ItemNotFoundException("Missing header '" + header + 
				"' in section '" + section + "' at line " + 
				ffp.getParsedLines() + " in file '" + filename + "'");
		}
		return value;
	}
	
	/**
		Get the value of a header as a list of sub-values. If the header is
		missing or empty an exception is thrown. Otherwise the 'split' 
		regular expression is used to the split the value into a list of values.
		
		@param ffp The flat file parser
		@param header The name of the header
		@param split A regular expression used to split the value
		@param section The name of the section that is being parsed
		@param filename The name of the file that is being parsed (used to 
			create the error message in case the header is missing)
		@return A list with the values of the header
	*/
	public List<String> getRequiredHeader(FlatFileParser ffp, String header, String split, 
		String section, String filename)
	{
		return Arrays.asList(getRequiredHeader(ffp, header, section, filename).split(split));
	}
	
	/**
		Get the index of value in a list of values or throw an exception
		if the value is not found. The lookup is case-insensitive.
		
		@param values The list of values to look in
		@param value The value to look for
		@param header The name of the header the values are from
		@param section The name of the section that is being parsed
		@param line The current line number in the parsed file
		@param filename The The name of the file that is being parsed (used to 
			create the error message in case the header is missing)
		@return The index of the value
	*/
	public int getRequiredIndex(List<String> values, String value, 
			String header, String section, int line, String filename)
	{
		int index = values.indexOf(value);
		if (index == -1 && value != null)
		{
			// Try case-insensitive lookup
			ListIterator<String> it = values.listIterator();
			while (it.hasNext())
			{
				if (value.equalsIgnoreCase(it.next()))
				{
					index = it.previousIndex();
					break;
				}
			}
		}
		if (index == -1) 
		{
			throw new ItemNotFoundException("Missing value '" + value + 
					"' for header '" + header + "' in section '" + section + 
					"' at line " + line + " in file '" + filename + "'");
		}
		return index;
	}


}
