/**
	$Id: MetadataModel.java 6875 2015-04-20 09:08:18Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

/**
	A model container of the contents of a BFS metadata file. This class
	is primarily intended to be used when parsing a BFS metadata file 
	(see {@link MetadataParser#parse(EventHandler)}), 
	but it can be used to programmatically create a BFS metadata file and 
	then write it out to a file (see {@link #print(Writer)}).
	<p>
	
	The model can contain multiple sections, each one with multiple
	entries. A section has a name, but it doesn't have to be unique.
	An entry is either a key/value pair or a key/array of values.
	There may be multiple entries with the same key in a section.
	
	@author Nicklas
	@version 2.15
	@base.modified $Date: 2015-04-20 11:08:18 +0200 (må, 20 apr 2015) $
*/
public class MetadataModel
	implements EventHandler
{

	private String subtype;
	private List<Section> sections;
	private Section files;
	private Section currentSection;
	
	/**
		Create a new metadata model.
	*/
	public MetadataModel()
	{
		this.sections = new ArrayList<Section>();
	}

	/*
		From the EventHandler interface
		-------------------------------
	*/
	@SuppressWarnings("rawtypes")
	@Override
	public void handleEvent(EventType eventType, Object eventData, BfsParser parser)
	{
		if (eventType == MetadataParser.BOF_EVENT)
		{
			setSubtype((String)eventData);
		}
		else if (eventType == MetadataParser.SECTION_EVENT)
		{
			currentSection = addSection((String)eventData);
		}
		else if (eventType == MetadataParser.ENTRY_EVENT)
		{
			currentSection.addEntry((SectionEntry)eventData);
		}
	}
	// ------------------------------------

	/**
		Utility method for printing the contents of this model object
		to a stream.
		@param out The output stream to write to
	*/
	public void print(Writer out)
	{
		MetadataWriter bfs = new MetadataWriter(out);
		bfs.setSubtype(getSubtype());
		for (int i = 0; i < getSectionCount(); ++i)
		{
			Section s = getSection(i);
			bfs.bfsPrintSection(s.getName(), false, false);
			for (int j = 0; j < s.getEntryCount(); ++j)
			{
				SectionEntry e = s.getEntry(j);
				bfs.bfsPrintValue(e.getKey(), e.getValues());
			}
		}
		bfs.flush();
	}
	
	/**
		Set the subtype of the BFS.
	*/
	public void setSubtype(String subtype)
	{
		this.subtype = subtype;
	}
	
	/**
		Get the BFS subtype.
	*/
	public String getSubtype()
	{
		return subtype;
	}
	
	/**
		Get the number of sections.
	*/
	public int getSectionCount()
	{
		return sections.size();
	}
	
	/**
		Adds a new section to the model.

		@param name The name of the section
		@return The new section
	*/
	public Section addSection(String name)
	{
		Section s = new Section(name);
		sections.add(s);
		if ("files".equals(name) && files == null) files = s;
		return s;
	}
	
	/**
		Get the section at a given index.
		@param index The index, a value between 0 and {@link #getSectionCount()}-1
		@return Information about the section
		@throws ArrayIndexOutOfBoundsException If the index is outside the 
			allowed range
	*/
	public Section getSection(int index)
	{
		return sections.get(index);
	}
	
	/**
		Get the first section with a given name. If there are multiple
		sections with the same name, only the first is returned.
		
		@param name The name of the section to find
		@return A section or null if no section exists
	*/
	public Section getSection(String name)
	{
		for (Section s : sections)
		{
			if (s.getName().equals(name)) return s;
		}
		return null;
	}
	
	/**
		Add an entry to the 'files' section.
		@param key The entry key
		@param name The filename (not verified)
	*/
	public void addFile(String key, String name)
	{
		if (files == null) addSection("files");
		files.addEntry(key, name);
	}
	
	/**
		Get the number of entries in the 'files' section.
	*/
	public int getFileCount()
	{
		if (files == null) return 0;
		return files.getEntryCount();
	}
	
	/**
		Get the file entry with a given index.
		@param index The index, a value between 0 and {@link #getFileCount()} - 1
		@return The file name
	*/
	public String getFile(int index)
	{
		if (files == null) return null;
		return files.getValue(index);
	}
	
	/**
		Get the file entry for a given key. If multiple entries with
		the same key exists, only the first is returned.
		@param key The key to look for
		@return The file name of null if no entry exists for the given key
	*/
	public String getFile(String key)
	{
		if (files == null) return null;
		return files.getValue(key);
	}
	
	/**
		Utility method for getting the value of the first entry
		in the first section that matches the given section and
		key name.
		@param section The name of the section
		@param key The key of the entry
		@return The value, or null if no section or entry is found
	*/
	public String getValue(String section, String key)
	{
		Section s = getSection(section);
		return s == null ? null : s.getValue(key);
	}

	/**
		Utility method for getting the values of the first entry
		in the first section that matches the given section and
		key name.
		@param section The name of the section
		@param key The key of the entry
		@return The values, or null if no section or entry is found
	*/
	public String[] getValues(String section, String key)
	{
		Section s = getSection(section);
		return s == null ? null : s.getValues(key);
	}

	
	/**
		Holds information about a section.
	*/
	public static class Section
	{
		private String name;
		private List<SectionEntry> entries;
		
		/**
			Create a new section.
			@param name The name of the section
		*/
		public Section(String name)
		{
			this.name = name;
			this.entries = new ArrayList<SectionEntry>();
		}
		
		/**
			Get the name of the section.
		*/
		public String getName()
		{
			return name;
		}
		
		/**
			Get the number of entries in this section.
		*/
		public int getEntryCount()
		{
			return entries.size();
		}
		
		/**
			Add a new entry to this section.
		*/
		public void addEntry(SectionEntry entry)
		{
			entries.add(entry);
		}
		
		/**
			Add a new entry to this section.
			@param key The key of the entry
			@param values The values
		*/
		public SectionEntry addEntry(String key, String... values)
		{
			SectionEntry e = new SectionEntry(key, values);
			addEntry(e);
			return e;
		}
		
		/**
			Get the entry in this section with the given index.
			@param index The index of the entry, a value between 0 and
				{@link #getEntryCount()} - 1
			@return A section entry
		*/
		public SectionEntry getEntry(int index)
		{
			return entries.get(index);
		}
		
		/**
			Get the first entry in this section with a given key. 
			@param key The key to look for
			@return A section entry, or null if no entry with the 
				given key exists
		*/
		public SectionEntry getEntry(String key)
		{
			for (SectionEntry e: entries)
			{
				if (e.getKey().equals(key)) return e;
			}
			return null;
		}
		
		/**
			Get the key of the entry at the given index.
			@param index The index of the entry, a value between 0 and
				{@link #getEntryCount()} - 1
			@return The key of the entry
		*/
		public String getKey(int index)
		{
			return getEntry(index).getKey();
		}
		
		/**
			Get the index of the first entry with the given key. 
			@param key The key to look for
			@return The index of the entry, or -1 if the key is not found
		*/
		public int getIndex(String key)
		{
			for (int i = 0; i < entries.size(); ++i)
			{
				if (entries.get(i).getKey().equals(key)) return i;
			}
			return -1;
		}
		
		/**
			Get the value of the entry at the given index.
			@param index The index of the entry, a value between 0 and
				{@link #getEntryCount()} - 1
			@return The value of the entry
		*/
		public String getValue(int index)
		{
			return getEntry(index).getValue();
		}

		/**
			Get the values of the entry at the given index.
			@param index The index of the entry, a value between 0 and
				{@link #getEntryCount()} - 1
			@return The value of the entry
		*/
		public String[] getValues(int index)
		{
			return getEntry(index).getValues();
		}
		
		/**
			Get the value of the first entry with a given key.
			@param key The key of the entry
			@return The value, or null if no entry for the given key exists
		*/
		public String getValue(String key)
		{
			SectionEntry e = getEntry(key);
			return e == null ? null : e.getValue();
		}

		/**
			Get the values of the first entry with a given key.
			@param key The key of the entry
			@return The value, or null if no entry for the given key exists
		*/
		public String[] getValues(String key)
		{
			SectionEntry e = getEntry(key);
			return e == null ? null : e.getValues();
		}

	}

	/**
		Holds information about a section entry.
	*/
	public static class SectionEntry
	{
		private String key;
		private String[] values;
		private String value;
		
		/**
			Create a new single-valued section entry.
			@param key The key of the entry
			@param value The value of the entry
		*/
		public SectionEntry(String key, String value)
		{
			this.key = key;
			this.value = value;
			this.values = new String[] { value };
		}
		
		/**
			Create a new section entry with an array of values.
			@param key The key of the entry
			@param values An array with the values
		*/
		public SectionEntry(String key, String... values)
		{
			this.key = key;
			this.values = values;
		}
		
		/**
			Get the key of the entry.
		*/
		public String getKey()
		{
			return key;
		}
		
		/**
			Get the number of values in the entry.
		*/
		public int getValueCount()
		{
			return values.length;
		}
		
		/**
			Get the value of the entry. If the entry is a
			multi-value entry, a single-value is created by concatenating
			the array of values using tab as a separator.
			@return The value of the entry
		*/
		public String getValue()
		{
			if (value == null)
			{
				String sep = "";
				StringBuilder sb = new StringBuilder();
				for (String s : values)
				{
					sb.append(sep);
					sb.append(s);
					sep = "\t";
				}
				value = sb.toString();
			}
			return value;
		}
		
		/**
			Get all values.
		*/
		public String[] getValues()
		{
			return values;
		}
		
	}
	
}
