/**
	$Id$

	Copyright (C) 2015 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.listable;

import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.Type;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

/**
	Source item transformer implementation that transform sample items to their
	child sample items. From the given set of sample ID:s the transformer will
	recursively load as many child samples as possible. The source items
	may optionally be included in the target set.

	@author Nicklas
	@since 3.5
*/
public class SampleToChildSampleTransformer 
	extends AbstractSourceItemTransformer
{

	private final boolean includeSourcesInTarget;
	
	/**
		Create a new sample to child sample transformer.
		@param includeSourcesInTarget TRUE to include the source items in the target
	*/
	public SampleToChildSampleTransformer(boolean includeSourcesInTarget) 
	{
		super(Item.SAMPLE, Item.SAMPLE);
		this.includeSourcesInTarget = includeSourcesInTarget;
	}
	
	@Override
	public Set<Integer> transform(TransformContext context, Set<Integer> source) 
	{
		DbControl dc = context.getDbControl();
		
		ItemQuery<Sample> query = Sample.getQuery();
		query.setIncludes(context.getInclude());
		// Samples with a sample as parent
		query.restrict(Restrictions.eq(Hql.property("parentType"), Expressions.integer(Item.SAMPLE.getValue())));
		query.join(Hql.innerJoin("creationEvent", "ce"));
		query.join(Hql.innerJoin("ce", "sources", "sbm"));
		query.join(Hql.innerJoin("sbm", "bioMaterial", "bm"));
		query.restrict(
			Restrictions.in(
				Hql.alias("bm"), 
				Expressions.parameter("parents")
			));

		// Keep track of all seen children and parents
		Set<Integer> allParents = new HashSet<Integer>();
		Set<Integer> allChildren = new HashSet<Integer>();
		if (includeSourcesInTarget) allChildren.addAll(source);

		// Parents that we have not yet checked
		Set<Integer> parents = source;
		while (parents.size() > 0)
		{
			query.setParameter("parents", parents, Type.INT);
			Set<Integer> children = safeIdList(dc, query, "parents", parents);
			
			// Store new children and "used" parents
			allChildren.addAll(children);
			allParents.addAll(parents);
			
			// Remove parents that we have already seen...
			// ...the remaining children become our new parents
			children.removeAll(allParents);
			parents = children;
		}
		
		return allChildren;
	}


}
