/**
	$Id: Fix.java 6939 2015-08-21 12:32:11Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview;


import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DataFileType;
import net.sf.basedb.core.FileSetMember;

/**
 	Holds information about a suggested fix to a failure.

	@author Nicklas
	@version 2.2
	@base.modified $Date: 2015-08-21 14:32:11 +0200 (fr, 21 aug 2015) $
	@see Failure#getFixes()
*/
public class Fix
{
	
	private String message;
	private BasicItem item;
	private AnnotationType annotationType;
	private Annotation annotation;
	private boolean fromInherited;
	private FileSetMember member;
	private DataFileType fileType;

	/**
		Create a new suggested fix for an item.
		@param message A message explaining what needs to be fixed
		@param item The item that needs to be edited by this fix
	*/
	public Fix(String message, BasicItem item)
	{
		this.message = message;
		this.item = item;
	}

	/**
		Create a new suggested fix for the annotation of an item.
		@param message A message explaining what needs to be fixed
		@param item The item that needs to be edited by this fix
		@param annotationType The annotation that needs to be edited by this fix or null if
			the fix doesn't include annotations
		@param fromInherited TRUE if the annotation is an inherited annotation or
			FALSE if it is a primary annotation
	*/
	public Fix(String message, BasicItem item, AnnotationType annotationType, boolean fromInherited)
	{
		this.message = message;
		this.item = item;
		this.annotationType = annotationType;
		this.fromInherited = fromInherited;
	}

	/**
		Create a new suggested fix for the annotation of an item.
		@param message A message explaining what needs to be fixed
		@param item The item that needs to be edited by this fix
		@param annotation The annotation that needs to be edited by this fix
		
		@since 3.6
	*/
	public Fix(String message, BasicItem item, Annotation annotation)
	{
		this.message = message;
		this.item = item;
		this.annotation = annotation;
		this.annotationType = annotation.getAnnotationType();
		this.fromInherited = annotation.getSource() != Annotation.Source.PRIMARY;
	}

	
	public Fix(String message, BasicItem item, FileSetMember member)
	{
		this.message = message;
		this.item = item;
		this.member = member;
	}
	
	public Fix(String message, BasicItem item, DataFileType fileType)
	{
		this.message = message;
		this.item = item;
		this.fileType = fileType;
	}
	
	/**
		Get a message explaining what needs to be fixed.
	*/
	public String getMessage()
	{
		return message;
	}
	
	/**
		Get the item that needs to be edited for this fix.
	*/
	public BasicItem getItem()
	{
		return item;
	}
	
	/**
		Get the annotation type of the annotation that needs to be edited
		for this fix.
		@return An annotation type or null if this is not a fix for annotations
	*/
	public AnnotationType getAnnotationType()
	{
		return annotationType;
	}

	/**
		Get the annotation that needs to be edited for this fix.
		@return An annotation or null if this is not a fix for annotations
		@since 3.6
	*/
	public Annotation getAnnotation()
	{
		return annotation;
	}
	
	/**
		If this fix involves editing an annotation or not.
		@see #isFromInherited()
	*/
	public boolean isAnnotationFix()
	{
		return annotationType != null;
	}
	
	/**
		If this fix involves editing an inherited annotation or a primary annotation.
		@see #isAnnotationFix()
	*/
	public boolean isFromInherited()
	{
		return fromInherited;
	}
	
	public FileSetMember getFileSetMember()
	{
		return member;
	}
	
	public DataFileType getDataFileType()
	{
		return fileType;
	}
	
	public boolean isDataFileFix()
	{
		return member != null || fileType != null;
	}
	
	
	@Override
	public String toString()
	{
		return getMessage();
	}
}
