/*
	$Id: TestUser.java 5827 2011-10-26 10:51:52Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.EnumSet;
import java.util.Set;

public class TestUser
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing users");
		write_header();
		// Standard tests: create, load, list
		int id2 = test_create(true);
		int id = test_create(false);
		test_load(id);
		test_list(-1);
		
		// Extra test: modify password
		test_change_password(id);
		test_duplicate(id);

		// Extra test: create an item and change the owner
 		int tag_id = TestTag.test_create("Test tag for setting owner", 0, false);
 		test_set_owner(id, Item.TAG, tag_id);
		
		// Extra tests: add, list and remove groups
		int groupId = TestGroup.test_create(true);
		test_listgroups(id, 0);
		TestGroup.test_adduser(groupId, id);
		test_listgroups(id, 1);
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestGroup.test_removeuser(groupId, id);
		test_listgroups(id, 0);
		TestGroup.test_delete(groupId);

		// Extra tests: add, list and remove roles
		int roleId = TestRole.test_create(true);
		test_listroles(id, 0);
		TestRole.test_adduser(roleId, id);
		test_listroles(id, 1);
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestRole.test_removeuser(roleId, id);
		test_listroles(id, 0);
		TestRole.test_delete(roleId);
		
		// Extra tests: add, list and remove projects
		int projectId = TestProject.test_create("User project", true);
		test_listprojects(id, 0);
		TestProject.test_adduser(projectId, id, EnumSet.of(Permission.WRITE));
		test_listprojects(id, 1);
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
		TestProject.test_removeuser(projectId, id);
		test_listprojects(id, 0);
		TestProject.test_delete(projectId);

		// Standard test: Delete
 		TestTag.test_delete(tag_id);
		test_delete(id);
		test_delete(id2);

		write("++Testing users "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.USER)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			String login = "user"+Application.generateRandomId(4);
			User u = User.getNew(dc, login, "password");
			if (setAll)
			{
				u.setName("Test user");
				u.setOrganisation("Lund university");
				u.setAddress("Sölvegaten 14A");
				u.setPhone("046-1111111");
				u.setFax("046-2222222");
				u.setEmail("user@thep.lu.se");
				u.setUrl("http://www.thep.lu.se");
				u.setDescription("Added at "+new Date());
				u.addToDefaultRolesAndGroups();
			}
			dc.saveItem(u);
			dc.commit();
			id = u.getId();
			write_item(0, u);
			write("--Create user OK");
		}
		catch (Throwable ex)
		{
			write("--Create user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			write_item(0, u);
			write("--Load user OK");
		}
		catch (Throwable ex)
		{
			write("--Load user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<User> l = User.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List users OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List users FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			dc.deleteItem(u);
			Set<ItemProxy> using = u.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(u + " is used by " + using);
			}
			dc.commit();
			write("--Delete user OK");
		}
		catch (Throwable ex)
		{
			write("--Delete user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tLogin\tDescription\tOrganisation\tAddress\tPhone\tFax\tEmail\tUrl");
			write("-- \t-- \t--------- \t-----\t-----------\t------------\t-------\t-----\t---\t-----\t---");
		}
	}
	static void write_item(int i, Group g)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+g.getId()+"\t"+g.getName()+"\t"+g.getDescription());
	}
	static void write_item(int i, Role r)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+r.getId()+"\t"+r.getName()+"\t"+r.getDescription());
	}
	static void write_item(int i, Project p)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+p.getId()+"\t"+p.getName()+"\t"+p.getDescription());
	}
	static void write_item(int i, User u)
		throws BaseException
	{
		if (!TestUtil.getSilent())
		{
			System.out.println(i+":\t"+u.getId()+"\t"+u.getName()+"\t"+u.getLogin()+"\t"+u.getDescription()+
				"\t"+u.getOrganisation()+","+u.getAddress()+","+u.getPhone()+","+u.getFax()+","+
				u.getEmail()+","+u.getUrl()
			);
		}

	}
	static void write(String message)
	{
		System.out.println(message);
	}

	static void test_change_password(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			u.setPassword("new password");
			dc.commit();
			write_item(0, u);
			write("--Change password OK");
		}
		catch (Throwable ex)
		{
			write("--Change password FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_duplicate(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			User duplicate = User.getNew(dc, u.getLogin(), "password");
			dc.saveItem(duplicate);
			dc.commit();
			dc = TestUtil.getDbControl();
			dc.deleteItem(duplicate);
			dc.commit();
			throw new BaseException("Expected DatabaseException");
		}
		catch (DatabaseException ex)
		{
			write("--Duplicate user OK");
		}
		catch (Throwable ex)
		{
			write("--Duplicate user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listroles(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			ItemResultList<Role> l = u.getRoles().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List roles for user OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List roles for user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listgroups(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			ItemResultList<Group> l = u.getGroups().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List groups for user OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List groups for user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_listprojects(int id, int expectedResults)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			ItemResultList<Project> l = u.getProjects().list(dc);
			for (int i=0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List projects for user OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List projects for user FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_set_owner(int id, Item itemType, int item_id)
	{
		if (id == 0 || item_id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			User u = User.getById(dc, id);
			Ownable o = (Ownable)itemType.getById(dc, item_id);
			o.setOwner(u);
			dc.commit();
			write("--Set owner OK");
		}
		catch (Throwable ex)
		{
			write("--Set owner FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

}

